// pipe1.cpp
// Glenn G. Chappell
// 13 Feb 2013
//
// For CS 321 Spring 2013
// Pipe Demo

#include <iostream>
using std::cout;
using std::endl;
#include <cstdlib>
using std::size_t;
#include <string>
using std::string;
#include <unistd.h>  // For pipe, fork, close, write, read, _exit


// main
// Open pipe, fork, child sends to parent thru pipe, parent prints
int main()
{
    int fd[2];     // File descriptors for pipe
                   // fd[1]: pipe input, open for writing
                   // fd[0]: pipe output, open for reading

    size_t dummy;  // For ignored return values

    // Attempt to make pipe
    int piperet = pipe(fd);
    if (piperet)  // No pipe?
    {
        // Error message, quit
        cout << "Pipe creation failed" << endl;
        _exit(1);
    }

    // Now we have a pipe
    //  (and we need to be sure to close both ends!)

    // Attempt to fork
    pid_t p = fork();
    if (p < 0)  // Fork failed?
    {
        // Error message, CLOSE PIPE, quit
        cout << "Fork failed" << endl;
        close(fd[0]);
        close(fd[1]);
        _exit(1);
    }

    // Now we have a pipe and 2 processes

    if (p)
    { // Parent
        const int BUFFSIZE = 1000;
        char buffer[BUFFSIZE];  // Buffer for message

        // Receive message
        size_t charcount = read(fd[0], buffer, BUFFSIZE);
            // Note that we limit the number of characters read,
            //  to the buffer size, so no overflow possible

        // Close our end of the pipe
        close(fd[0]);

        // Notify user of receive & print message
        cout << "PARENT: I have a message from my child!" << endl;
        cout << "PARENT: It says:" << endl << endl;
        dummy = write(1, buffer, charcount);
        cout << endl << endl;
    }
    else
    { // Child
        // Notify user of send
        cout << "CHILD: I am sending a message to my parent."
             << endl << endl;

        // Send message to parent
        string msg = "Hello, O most honored parent!";
        dummy = write(fd[1], msg.c_str(), msg.size());

        // Close our end of the pipe, quit
        close(fd[1]);
        _exit(0);
    }

    // Only parent executes this
    cout << "PARENT: All done" << endl;

    return 0;
}

